@@ -2,12 +2,8 @@
  * GNOME time/date display module.
  * (C) 1997 The Free Software Foundation
  *
- * Authors: Miguel de Icaza
- *          Federico Mena
- *          Stuart Parmenter
- *          Alexander Larsson
- *
- * Feel free to implement new look and feels :-)
+ * Authors: George Lebl
+ * 	    Gediminas Paulauskas
  */
 
 #include "config.h"
@@ -18,6 +14,7 @@
 #include <dirent.h>
 #include <string.h>
 #include <time.h>
+#include <langinfo.h>
 
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
@@ -30,11 +27,12 @@
 
 #include "clock.h"
 
-#define INTERNETSECOND (864/4)
+#define INTERNETSECOND (864)
+#define INTERNETBEAT   (86400)
 
 static const char* KEY_HOUR_FORMAT	= "hour_format";
+static const char* KEY_SHOW_SECONDS 	= "show_seconds";
 static const char* KEY_SHOW_DATE 	= "show_date";
-static const char* KEY_SHOW_TOOLTIP	= "show_tooltip";
 static const char* KEY_GMT_TIME		= "gmt_time";
 static const char* KEY_UNIX_TIME	= "unix_time";
 static const char* KEY_INTERNET_TIME	= "internet_time";
@@ -43,33 +41,33 @@ extern GtkTooltips *panel_tooltips;
 
 typedef struct _ClockData ClockData;
 
-typedef void (*ClockUpdateFunc) (ClockData *, time_t);
-
 struct _ClockData {
+	/* widgets */
 	GtkWidget *applet;
 	GtkWidget *clockw;
+	GtkWidget *props;
   
-	guint timeout;
-	int timeouttime;
+	/* preferences */
 	int hourformat;
-        gboolean showdate;
+	gboolean showseconds;
+	gboolean showdate;
 	gboolean unixtime;
 	gboolean internettime;
 	gboolean gmt_time;
-	gboolean showtooltip;
-	ClockUpdateFunc update_func;
+	
+	/* runtime data */
+	char *timeformat;
+	guint timeout;
+	int timeouttime;
 	PanelAppletOrient orient;
 	int size;
-
-	GtkWidget *props;
 };
 
-typedef struct {
-	GtkWidget *time; /*the time label*/
-	GtkWidget *align; /* Needed for changing the padding */
-} ComputerClock;
-
+static void update_clock (ClockData * cd, time_t current_time);
 
+static void set_atk_name_description (GtkWidget *widget,
+                                      const char *name,
+                                      const char *desc);
 static void display_properties_dialog (BonoboUIComponent *uic,
 				       ClockData         *cd,
 				       const gchar       *verbname);
@@ -88,31 +86,37 @@ clock_timeout_callback (gpointer data)
 
 	time (&current_time);
 
-	(*cd->update_func) (cd, current_time);
+	update_clock (cd, current_time);
 
-	if (cd->unixtime) {
-		if (cd->timeouttime != 1000) {
-			cd->timeouttime = 1000;
-			cd->timeout = g_timeout_add (cd->timeouttime,
-						     clock_timeout_callback,
-						     cd);
-			return FALSE;
-		}
-	} else if(cd->internettime) {
-		if (cd->timeouttime != INTERNETSECOND) {
-			cd->timeouttime = INTERNETSECOND;
-			cd->timeout = g_timeout_add (cd->timeouttime,
-						     clock_timeout_callback,
-						     cd);
-			return FALSE;
-		}	   
-	} else {
-		if (cd->timeouttime != 36000) {
-			cd->timeouttime = 36000;
-			cd->timeout = g_timeout_add (cd->timeouttime,
-						     clock_timeout_callback,
-						     cd);
-			return FALSE;
+	if (!cd->showseconds && !cd->unixtime) {
+		if (!cd->internettime) {
+			int sec = current_time % 60;
+			if (sec != 0 || cd->timeouttime != 60000) {
+				/* ensure next update is exactly on 0 seconds */
+				cd->timeouttime = (60 - sec)*1000;
+				cd->timeout = g_timeout_add (cd->timeouttime,
+							     clock_timeout_callback,
+							     cd);
+				return FALSE;
+			}
+		} else {
+			struct tm *tm;
+			time_t bmt;
+			long isec;
+
+			/* BMT (Biel Mean Time) is GMT+1 */
+			bmt = current_time + 3600;
+			tm = gmtime (&bmt);
+			isec = ((tm->tm_hour*3600 + tm->tm_min*60 + tm->tm_sec)*10) % 864;
+			
+			if (isec != 0 || cd->timeouttime != INTERNETBEAT) {
+				/* ensure next update is exactly on beat limit */
+				cd->timeouttime = (864 - isec)*100;
+				cd->timeout = g_timeout_add (cd->timeouttime,
+							     clock_timeout_callback,
+							     cd);
+				return FALSE;
+			}
 		}
 	}
 
@@ -126,25 +130,109 @@ get_itime (time_t current_time)
 	float itime;
 	time_t bmt;
 
+	/* BMT (Biel Mean Time) is GMT+1 */
 	bmt = current_time + 3600;
 	tm = gmtime (&bmt);
-	itime = (tm->tm_hour*60 + tm->tm_min + tm->tm_sec/60.0)*1000.0/1440.0;
+	itime = (tm->tm_hour*3600.0 + tm->tm_min*60.0 + tm->tm_sec)/86.4;
 
 	return itime;
 }
 
 static void
-computer_clock_update_func (ClockData * cd,
-			    time_t current_time)
+update_timeformat(ClockData *cd)
+{
+	const char *time;
+
+	if (cd->hourformat == 12) {
+		if (cd->showseconds)
+			time = _("%I:%M:%S %p");
+		else
+			time = _("%I:%M %p");
+	} else {
+		if (cd->showseconds)
+			time = _("%H:%M:%S");
+		else
+			time = _("%H:%M");
+	}
+
+	g_free (cd->timeformat);
+	if (cd->showdate) {
+		/* Show date in another line if panel is vertical, or
+		 * horizontal but large enough to hold two lines of text */
+		if (cd->orient == PANEL_APPLET_ORIENT_LEFT ||
+		    cd->orient == PANEL_APPLET_ORIENT_RIGHT ||
+		    cd->size >= GNOME_Vertigo_PANEL_MEDIUM)
+			cd->timeformat = g_strconcat (time, "\n",
+					              _("%a %b %d"), NULL);
+		else
+			cd->timeformat = g_strconcat (time, " ",
+					              _("%a %b %d"), NULL);
+	} else {
+		cd->timeformat = g_strdup (time);
+	}
+}
+
+/* sets accessible name and description for the widget */
+static void
+set_atk_name_description (GtkWidget  *widget,
+			  const char *name,
+			  const char *desc)
+{
+	AtkObject *obj;
+	obj = gtk_widget_get_accessible (widget);
+
+	/* return if gail is not loaded */
+	if (!GTK_IS_ACCESSIBLE (obj))
+		return;
+
+	if (desc != NULL)
+		atk_object_set_description (obj, desc);
+	if (name != NULL)
+		atk_object_set_name (obj, name);
+} 
+
+/* sets up ATK relation between the widgets */
+static void
+add_atk_relation (GtkWidget       *widget,
+		  GSList          *list,
+		  AtkRelationType  type)
+{
+	AtkRelationSet *relation_set;
+	AtkObject      *aobj;
+
+	aobj = gtk_widget_get_accessible (widget);
+	if (!GTK_IS_ACCESSIBLE (aobj))
+		return;
+
+	relation_set = atk_object_ref_relation_set (aobj);
+
+	if (list) {
+		AtkObject  **accessible_array;
+		AtkRelation *relation;
+		GSList      *l;
+		guint        len;
+		int          i = 0;
+
+		len = g_slist_length (list);
+		accessible_array =
+			(AtkObject **)g_malloc (sizeof (AtkObject *) * len);
+
+		for (l = list, i = 0; l; l = l->next, i++)
+			accessible_array [i] = gtk_widget_get_accessible (l->data);
+
+		relation = atk_relation_new (accessible_array, len, type);
+		atk_relation_set_add (relation_set, relation);
+		g_object_unref (relation);
+	}
+}
+
+static void
+update_clock (ClockData * cd, time_t current_time)
 {
-	ComputerClock *cc;
 	struct tm *tm;
-	GString *gs = g_string_new("");
 	char date[256], hour[256];
 	char *utf8;
 	
-	cc = g_object_get_data (G_OBJECT (cd->clockw), "cc");
-
 	if (cd->gmt_time)
 		tm = gmtime (&current_time);
 	else
@@ -160,84 +248,28 @@ computer_clock_update_func (ClockData * cd,
 		} else {
 			g_snprintf (hour, sizeof(hour), "%lu", (unsigned long)current_time);
 		} 
-		g_string_append (gs, hour);
 	} else if (cd->internettime) {
 		float itime = get_itime (current_time);
-		g_snprintf (hour, sizeof(hour), "@%3.2f", itime);
-		g_string_append (gs, hour);
-	} else if (cd->hourformat == 12) {
-		/* This format string is used, to display the actual time in
-		   12 hour format.  */
-		if ((cd->orient == PANEL_APPLET_ORIENT_LEFT ||
-		     cd->orient == PANEL_APPLET_ORIENT_RIGHT) &&
-		    cd->size >= GNOME_Vertigo_PANEL_MEDIUM) {
-			if (strftime (hour, sizeof (hour), _("%I:%M\n%p"), tm) <= 0)
-				strcpy (hour, "???");
-		} else {
-			if (strftime (hour, sizeof (hour), _("%I:%M %p"), tm) <= 0)
-				strcpy (hour, "???");
-		} 
-		g_string_append (gs, hour);
-	} else if (cd->hourformat == 24) {
-		/* This format string is used, to display the actual time in
-		   24 hour format.  */
-		if (strftime (hour, sizeof (hour), _("%H:%M"), tm) <= 0)
-			strcpy (hour, "???");
-		g_string_append (gs,hour);
-	}
-
-	if (cd->showdate && !cd->unixtime && !cd->internettime) {
-		if ((cd->orient == PANEL_APPLET_ORIENT_LEFT ||
-		     cd->orient == PANEL_APPLET_ORIENT_RIGHT) &&
-		    cd->size >= GNOME_Vertigo_PANEL_MEDIUM) {
-			/* This format string is used, to display the actual day,
-			   when showing a vertical panel.  For an explanation of
-			   this format string type 'man strftime'.  */
-			if (strftime (date, sizeof (date), _("%a\n%b %d"), tm) <= 0)
-				strcpy (date, "???");
-		} else {
-			/* This format string is used, to display the actual day,
-			   when showing a horizontal panel.  */
-			if (strftime (date, sizeof (date), _("%a %b %d"), tm) <= 0)
-				strcpy (date, "???");
-		}
-		if (cd->size < GNOME_Vertigo_PANEL_MEDIUM)
-			g_string_append_c (gs,' ');
+		if (cd->showseconds)
+			g_snprintf (hour, sizeof (hour), "@%3.2f", itime);
 		else
-			g_string_append_c (gs,'\n');
-		g_string_append (gs, date);
+			g_snprintf (hour, sizeof (hour), "@%3.0f", itime);
+	} else {
+		if (strftime (hour, sizeof (hour), cd->timeformat, tm) <= 0)
+			strcpy (hour, "???");
 	}
 
-	/* Set the applets tooltip */
-	if (cd->showtooltip && !cd->unixtime && !cd->internettime) {
-		if (strftime (date, sizeof (date), _("%A, %B %d"), tm) <= 0)
-			strcpy (date, "???");
-		utf8 = g_locale_to_utf8 (date, -1, NULL, NULL, NULL);
-  		gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (cd->applet),
-				      utf8, NULL);
-		g_free (utf8);
-	}
-	
-	/*if we are vertical, just make it char per line */
-	if ((cd->orient == PANEL_APPLET_ORIENT_LEFT ||
-	     cd->orient == PANEL_APPLET_ORIENT_RIGHT) &&
-	    cd->size < GNOME_Vertigo_PANEL_MEDIUM) {
-		char *p;
-		GString *gst = g_string_new ("");
-		
-		for(p = gs->str; *p; p++) {
-			if (p != gs->str)
-				g_string_append_c (gst, '\n');
-			g_string_append_c (gst, *p);
-		}
-		g_string_free (gs, TRUE);
-		gs = gst;
-	}
+	utf8 = g_locale_to_utf8 (hour, -1, NULL, NULL, NULL);
+	gtk_label_set_text (GTK_LABEL (cd->clockw), utf8);
+	g_free (utf8);
 
-	utf8 = g_locale_to_utf8 (gs->str, -1, NULL, NULL, NULL);
+	/* Show date in tooltip */
+	if (strftime (date, sizeof (date), _("%A %B %d"), tm) <= 0)
+		strcpy (date, "???");
 
-	gtk_label_set_text (GTK_LABEL (cc->time), utf8);
-	g_string_free (gs, TRUE);
+	utf8 = g_locale_to_utf8 (date, -1, NULL, NULL, NULL);
+	gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (cd->applet),
+			      utf8, NULL);
 	g_free (utf8);
 }
 
@@ -247,87 +279,59 @@ refresh_clock (ClockData *cd)
 	time_t current_time;
 	
 	time (&current_time);
-	(*cd->update_func) (cd, current_time);
+	update_clock (cd, current_time);
 }
 
 static void
 refresh_clock_timeout(ClockData *cd)
 {
 	time_t current_time;
-	struct tm *tm;
 	
+	update_timeformat (cd);
+
 	if (cd->timeout)
 		g_source_remove (cd->timeout);
 
 	time (&current_time);
-	(*cd->update_func) (cd, current_time);
+	update_clock (cd, current_time);
 	
-	/* Install timeout handler */
-	if (cd->gmt_time)
-		tm = gmtime (&current_time);
-	else
-		tm = localtime (&current_time);
+	if (cd->internettime) {
+		if (cd->showseconds)
+			cd->timeouttime = INTERNETSECOND;
+		else {
+			struct tm *tm;
+			time_t bmt;
+			long isec;
 
-	if(cd->unixtime)
+			/* BMT (Biel Mean Time) is GMT+1 */
+			bmt = current_time + 3600;
+			tm = gmtime (&bmt);
+			isec = ((tm->tm_hour*3600 + tm->tm_min*60 + tm->tm_sec)*10) % 864;
+			cd->timeouttime = (864 - isec)*100;
+		}
+	}
+	else if(cd->unixtime || cd->showseconds)
 		cd->timeouttime = 1000;
-        else if (cd->internettime)
-                cd->timeouttime = INTERNETSECOND;
 	else
-		cd->timeouttime = 36000-(tm->tm_sec*600);
+		cd->timeouttime = (60 - current_time % 60)*1000;
 	
 	cd->timeout = g_timeout_add (cd->timeouttime,
-				     clock_timeout_callback,
-				     cd);
+	                             clock_timeout_callback,
+	                             cd);
 }
 
 static void
-create_computer_clock_widget (GtkWidget ** clock, ClockUpdateFunc * update_func)
+destroy_clock(GtkWidget * widget, ClockData *cd)
 {
-	GtkWidget *frame;
-	GtkWidget *vbox;
-	ComputerClock *cc;
-
-	frame = gtk_frame_new (NULL);
-	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
-	gtk_widget_show (frame);
-
-	cc = g_new0 (ComputerClock, 1);
-
-	cc->align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-	gtk_container_set_border_width (GTK_CONTAINER (cc->align), 3);
-	gtk_container_add (GTK_CONTAINER (frame), cc->align);
-	gtk_widget_show (cc->align);
-
-	vbox = gtk_vbox_new (FALSE, FALSE);
-	gtk_container_add (GTK_CONTAINER (cc->align), vbox);
-	gtk_widget_show (vbox);
-
-	cc->time = gtk_label_new ("hmm?");
-	gtk_label_set_justify (GTK_LABEL (cc->time), GTK_JUSTIFY_CENTER);
-
-	gtk_box_pack_start_defaults (GTK_BOX (vbox), cc->time);
-	gtk_widget_show (cc->time);
-
-	g_object_set_data_full (G_OBJECT (frame), "cc", cc, g_free);
-
-	*clock = frame;
-	*update_func = computer_clock_update_func;
-}
-
-static void
-destroy_clock(GtkWidget * widget, gpointer data)
-{
-	ClockData *cd = data;
-
-	if (cd->timeout > 0)
+	if (cd->timeout > 0) {
 		g_source_remove (cd->timeout);
-	
-	cd->timeout = 0;
+		cd->timeout = 0;
+	}
 
-	if(cd->props)
+	if (cd->props) {
 		gtk_widget_destroy (cd->props);
-	
-	cd->props = NULL;
+		cd->props = NULL;
+	}
 
 	g_free (cd);
 }
@@ -337,8 +341,10 @@ create_clock_widget (ClockData *cd)
 {
 	GtkWidget *clock;
 
-	/*FIXME: different clock types here */
-	create_computer_clock_widget (&clock, &cd->update_func);
+	clock = gtk_label_new ("hmm?");
+	gtk_label_set_justify (GTK_LABEL (clock), GTK_JUSTIFY_CENTER);
+	gtk_label_set_line_wrap (GTK_LABEL (clock), TRUE);
+	gtk_widget_show (clock);
 
 	cd->clockw = clock;
 
@@ -351,6 +357,9 @@ create_clock_widget (ClockData *cd)
 			  G_CALLBACK (destroy_clock),
 			  cd);
 	
+	set_atk_name_description (GTK_WIDGET (cd->applet), NULL,
+	                          _("Computer Clock"));
+
 	/* Refresh the clock so that it paints its first state */
 	refresh_clock_timeout(cd);
 }
@@ -366,7 +375,7 @@ applet_change_orient (PanelApplet       *applet,
 
 	time (&current_time);
 	cd->orient = orient;
-	(*cd->update_func) (cd, current_time);
+	update_clock (cd, current_time);
 }
 
 static void
@@ -400,16 +409,12 @@ applet_change_pixel_size (PanelApplet *applet,
 			  ClockData   *cd)
 {
 	time_t current_time;
-	ComputerClock *cc;
-
-	/* Adjust the padding on the text for small sizes of the panel (so the widget doesn't become bigger than the panel */
-	cc = g_object_get_data (G_OBJECT (cd->clockw), "cc");
-	gtk_container_set_border_width (GTK_CONTAINER (cc->align),
-					((size < GNOME_Vertigo_PANEL_SMALL)? 0 : 3 ) );
 	
 	time (&current_time);
 	cd->size = size;
-	(*cd->update_func) (cd, current_time);
+
+	update_timeformat (cd);
+	update_clock (cd, current_time);
 }
 
 
@@ -419,30 +424,40 @@ copy_time (BonoboUIComponent *uic,
 	   const gchar       *verbname)
 {
 	time_t current_time = time (NULL);
-	struct tm *tm;
 	char string[256];
 	char *utf8;
 
-	if (cd->gmt_time)
-		tm = gmtime (&current_time);
-	else
-		tm = localtime (&current_time);
-
 	if (cd->unixtime) {
 		g_snprintf (string, sizeof(string), "%lu",
 			    (unsigned long)current_time);
 	} else if (cd->internettime) {
 		float itime = get_itime (current_time);
-		g_snprintf (string, sizeof (string), "@%3.2f", itime);
-	} else if (cd->hourformat == 12) {
-		/* This format string is used, to display the actual time in
-		   12 hour format.  */
-		if (strftime (string, sizeof (string), _("%I:%M %p"), tm) <= 0)
-			strcpy (string, "???");
-	} else if (cd->hourformat == 24) {
-		/* This format string is used, to display the actual time in
-		   24 hour format.  */
-		if (strftime (string, sizeof (string), _("%H:%M"), tm) <= 0)
+		if (cd->showseconds)
+			g_snprintf (string, sizeof (string), "@%3.2f", itime);
+		else
+			g_snprintf (string, sizeof (string), "@%3.0f", itime);
+	} else {
+		const char *format;
+		struct tm *tm;
+
+		if (cd->hourformat == 12) {
+			if (cd->showseconds)
+				format = _("%I:%M:%S %p");
+			else
+				format = _("%I:%M %p");
+		} else {
+			if (cd->showseconds)
+				format = _("%H:%M:%S");
+			else
+				format = _("%H:%M");
+		}
+
+		if (cd->gmt_time)
+			tm = gmtime (&current_time);
+		else
+			tm = localtime (&current_time);
+
+		if (strftime (string, sizeof (string), format, tm) <= 0)
 			strcpy (string, "???");
 	}
 
@@ -478,19 +493,19 @@ copy_date (BonoboUIComponent *uic,
 
 /* current timestamp */
 static const BonoboUIVerb clock_menu_verbs [] = {
-	BONOBO_UI_UNSAFE_VERB ("ClockProperties", display_properties_dialog),
-	BONOBO_UI_UNSAFE_VERB ("ClockHelp",       display_help_dialog),
-	BONOBO_UI_UNSAFE_VERB ("ClockAbout",      display_about_dialog),
-	BONOBO_UI_UNSAFE_VERB ("ClockCopyTime",   copy_time),
-	BONOBO_UI_UNSAFE_VERB ("ClockCopyDate",   copy_date),
-        BONOBO_UI_VERB_END
+	BONOBO_UI_UNSAFE_VERB ("ClockPreferences", display_properties_dialog),
+	BONOBO_UI_UNSAFE_VERB ("ClockHelp",        display_help_dialog),
+	BONOBO_UI_UNSAFE_VERB ("ClockAbout",       display_about_dialog),
+	BONOBO_UI_UNSAFE_VERB ("ClockCopyTime",    copy_time),
+	BONOBO_UI_UNSAFE_VERB ("ClockCopyDate",    copy_date),
+	BONOBO_UI_VERB_END
 };
 
 static void
 hour_format_changed (GConfClient  *client,
-		     guint         cnxn_id,
-		     GConfEntry   *entry,
-		     ClockData *clock)
+                     guint         cnxn_id,
+                     GConfEntry   *entry,
+                     ClockData    *clock)
 {
 	int value;
 	
@@ -504,14 +519,32 @@ hour_format_changed (GConfClient  *client,
 	else
 		clock->hourformat = 12;
 
+	update_timeformat (clock);
 	refresh_clock (clock);
 }
 
+static void
+show_seconds_changed (GConfClient  *client,
+                   guint         cnxn_id,
+                   GConfEntry   *entry,
+                   ClockData    *clock)
+{
+	gboolean value;
+	
+	if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
+		return;
+
+	value = gconf_value_get_bool (entry->value);
+	
+	clock->showseconds = (value != 0);
+	refresh_clock_timeout (clock);
+}
+
 static void
 show_date_changed (GConfClient  *client,
-		   guint         cnxn_id,
-		   GConfEntry   *entry,
-		   ClockData *clock)
+                   guint         cnxn_id,
+                   GConfEntry   *entry,
+                   ClockData    *clock)
 {
 	gboolean value;
 	
@@ -521,14 +554,15 @@ show_date_changed (GConfClient  *client,
 	value = gconf_value_get_bool (entry->value);
 	
 	clock->showdate = (value != 0);
+	update_timeformat (clock);
 	refresh_clock (clock);
 }
 
 static void
-internet_time_changed (GConfClient  *client,
-		       guint         cnxn_id,
-		       GConfEntry   *entry,
-		       ClockData *clock)
+internet_time_changed (GConfClient *client,
+                       guint        cnxn_id,
+                       GConfEntry  *entry,
+                       ClockData   *clock)
 {
 	gboolean value;
 	
@@ -542,10 +576,10 @@ internet_time_changed (GConfClient  *client,
 }
 
 static void
-unix_time_changed (GConfClient  *client,
-		  guint         cnxn_id,
-		  GConfEntry   *entry,
-		  ClockData *clock)
+unix_time_changed (GConfClient *client,
+                   guint        cnxn_id,
+                   GConfEntry  *entry,
+                   ClockData   *clock)
 {
 	gboolean value;
 	
@@ -560,9 +594,9 @@ unix_time_changed (GConfClient  *client,
 
 static void
 gmt_time_changed (GConfClient  *client,
-		  guint         cnxn_id,
-		  GConfEntry   *entry,
-		  ClockData *clock)
+                  guint         cnxn_id,
+                  GConfEntry   *entry,
+                  ClockData    *clock)
 {
 	gboolean value;
 	
@@ -575,23 +609,6 @@ gmt_time_changed (GConfClient  *client,
 	refresh_clock_timeout (clock);
 }
 
-static void
-show_tooltip_changed (GConfClient  *client,
-		      guint         cnxn_id,
-		      GConfEntry   *entry,
-		      ClockData *clock)
-{
-	gboolean value;
-	
-	if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
-		return;
-
-	value = gconf_value_get_bool (entry->value);
-	
-	clock->showtooltip = (value != 0);
-	refresh_clock (clock);
-}
-
 static void
 setup_gconf (ClockData *clock)
 {
@@ -609,17 +626,17 @@ setup_gconf (ClockData *clock)
 	g_free (key);
 
 	key = panel_applet_gconf_get_full_key (PANEL_APPLET (clock->applet),
-					       KEY_SHOW_DATE);
+					       KEY_SHOW_SECONDS);
 	gconf_client_notify_add(client, key,
-				(GConfClientNotifyFunc)show_date_changed,
+				(GConfClientNotifyFunc)show_seconds_changed,
 				clock,
 				NULL, NULL);
 	g_free (key);
 
 	key = panel_applet_gconf_get_full_key (PANEL_APPLET (clock->applet),
-					       KEY_SHOW_TOOLTIP);
+					       KEY_SHOW_DATE);
 	gconf_client_notify_add(client, key,
-				(GConfClientNotifyFunc)show_tooltip_changed,
+				(GConfClientNotifyFunc)show_date_changed,
 				clock,
 				NULL, NULL);
 	g_free (key);
@@ -655,7 +672,7 @@ fill_clock_applet(PanelApplet *applet)
 	ClockData *cd;
 	GError *error;
 	
-	panel_applet_add_preferences (applet, "/schemas/apps/clock-applet/prefs", NULL);
+	panel_applet_add_preferences (applet, "/schemas/apps/clock_applet/prefs", NULL);
 	
 	cd = g_new0 (ClockData, 1);
 
@@ -665,29 +682,17 @@ fill_clock_applet(PanelApplet *applet)
 
 	error = NULL;
 	cd->hourformat = panel_applet_gconf_get_int (applet, KEY_HOUR_FORMAT, &error);
-	if (error) {
-		/* Do NOT translate the clock/hourformat= part.  What you
-		 * should change is the number 12.  If your country code should use
-		 * 12 hour format by default, leave it at 12, otherwise use 24
-		 * for 24 hour format.  Those are the only two supported */
-		char *transl = _("clock/hourformat=12");
-		const int len = strlen ("clock/hourformat=");
-		/* sanity */
-		if (strncmp (transl, "clock/hourformat=", len)) {
-			g_warning("Clock applet string clock/hourformat=12 "
-				  "was not translated correctly in this locale.");
-			cd->hourformat = 12;
-		} else {
-			cd->hourformat = atoi (transl + len);
-		}
+	if (error || (cd->hourformat != 12 && cd->hourformat != 24)) {
+		/* if value is not valid, set it according to locale */
+		const char *am = nl_langinfo (AM_STR);
+		cd->hourformat = (am[0] == '\0') ? 24 : 12;
 
-		g_error_free (error);
+		if (error)
+			g_error_free (error);
 	}
 
-	/* ensure value is valid */
-	if (cd->hourformat != 12 && cd->hourformat != 24)
-		cd->hourformat = 12;
-
+	cd->showseconds = panel_applet_gconf_get_bool (applet, KEY_SHOW_SECONDS, NULL);
+	
 	error = NULL;
 	cd->showdate = panel_applet_gconf_get_bool (applet, KEY_SHOW_DATE, &error);
 	if (error) {
@@ -699,19 +704,15 @@ fill_clock_applet(PanelApplet *applet)
 			cd->showdate = TRUE;
 	}
 
-	error = NULL;
-	cd->showtooltip = panel_applet_gconf_get_bool (applet, KEY_SHOW_TOOLTIP, &error);
-	if (error) {
-		g_error_free (error);
-		cd->showtooltip = TRUE; /* Default value*/
-	}
-
 	cd->gmt_time = panel_applet_gconf_get_bool (applet, KEY_GMT_TIME, NULL);
 	cd->unixtime = panel_applet_gconf_get_bool (applet, KEY_UNIX_TIME, NULL);
 	cd->internettime = panel_applet_gconf_get_bool (applet, KEY_INTERNET_TIME, NULL);
 
+	cd->timeformat = NULL;
+
 	create_clock_widget (cd);
 
+	gtk_container_set_border_width (GTK_CONTAINER (cd->applet), 4);
 	gtk_container_add (GTK_CONTAINER (cd->applet), cd->clockw);
 
 	gtk_widget_show (cd->applet);
@@ -767,6 +768,16 @@ set_hour_format_cb (GtkWidget *w,
 	}
 }
 
+static void
+set_show_seconds_cb (GtkWidget *w,
+                     ClockData *clock)
+{
+	panel_applet_gconf_set_bool (PANEL_APPLET (clock->applet),
+				     KEY_SHOW_SECONDS,
+				     GTK_TOGGLE_BUTTON (w)->active,
+				     NULL);
+}
+
 static void
 set_show_date_cb (GtkWidget *w,
 		  ClockData *clock)
@@ -807,16 +818,6 @@ set_gmt_time_cb (GtkWidget *w,
 				     NULL);
 }
 
-static void
-set_show_tooltip_cb (GtkWidget * w,
-		     ClockData *clock)
-{
-	panel_applet_gconf_set_bool (PANEL_APPLET (clock->applet),
-				     KEY_SHOW_TOOLTIP,
-				     GTK_TOGGLE_BUTTON (w)->active,
-				     NULL);
-}
-
 static void
 properties_response_cb (GtkWidget *widget, gint id, gpointer data)
 {
@@ -842,188 +843,186 @@ display_properties_dialog (BonoboUIComponent *uic,
 			   ClockData         *cd,
 			   const gchar       *verbname)
 {
-        GtkWidget *hbox;
-        GtkWidget *vbox;
-        GtkWidget *hour_frame;
-	GtkWidget *table;
+	GtkWidget *hbox;
+	GtkWidget *hour_frame;
+	GtkWidget *type_box;
+	GtkWidget *options_frame;
+	GtkWidget *vbox;
 	GtkWidget *twelvehour;
 	GtkWidget *twentyfourhour;
+	GtkWidget *showseconds;
 	GtkWidget *showdate;
-	GtkWidget *showtooltip;
 	GtkWidget *unixtime;
 	GtkWidget *internettime;
 	GtkWidget *use_gmt_time;
-	GdkPixbuf *pixbuf;
-	gchar *file;
+	GSList    *list;
+	char      *file;
 
 	if (cd->props != NULL) {
 		gtk_window_present (GTK_WINDOW (cd->props));
 		return;
 	}
 
-	cd->props = gtk_dialog_new_with_buttons (_("Clock Properties"), NULL, 0,
+	cd->props = gtk_dialog_new_with_buttons (_("Clock Preferences"), NULL, 0,
 						 GTK_STOCK_HELP,
 						 GTK_RESPONSE_HELP,
 						 GTK_STOCK_CLOSE,
 						 GTK_RESPONSE_CLOSE,
 						 NULL);
 
-	pixbuf = NULL;
-	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-clock.png", TRUE, NULL);
-	if (!file) {
-		g_warning (G_STRLOC ": gnome-clock.png cannot be found");
+	gtk_dialog_set_has_separator (GTK_DIALOG (cd->props), FALSE);
+	gtk_dialog_set_default_response (GTK_DIALOG (cd->props), GTK_RESPONSE_CLOSE);
+		
+	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
+	                                  "gnome-clock.png", TRUE, NULL);
+	if (file) {
+		GdkPixbuf *pixbuf;
 		pixbuf = gdk_pixbuf_new_from_file (file, NULL);
-	}
-
-	if (pixbuf) {
 		gtk_window_set_icon (GTK_WINDOW (cd->props), pixbuf);
 		g_object_unref (pixbuf);
+		g_free (file);
 	}
+	else
+		g_warning (G_STRLOC ": gnome-clock.png cannot be found");
 
-	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
+	hbox = gtk_hbox_new (FALSE, 3);
+	gtk_widget_show (hbox);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (cd->props)->vbox), hbox, FALSE, FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 3);
 	
-	hour_frame = gtk_frame_new (_("Time Format"));
-	gtk_container_set_border_width (GTK_CONTAINER (hour_frame), GNOME_PAD);
+	hour_frame = gtk_frame_new (_("Clock type"));
 	gtk_box_pack_start (GTK_BOX (hbox), hour_frame, FALSE, FALSE, 0);
+	gtk_widget_show (hour_frame);
+	gtk_container_set_border_width (GTK_CONTAINER (hour_frame), 3);
 
-	table = gtk_table_new (2, 2, FALSE);
-	gtk_container_add (GTK_CONTAINER (hour_frame), table);
-	gtk_widget_show (table);
+	type_box = gtk_vbox_new (FALSE, 3);
+	gtk_container_add (GTK_CONTAINER (hour_frame), type_box);
+	gtk_widget_show (type_box);
+	gtk_container_set_border_width (GTK_CONTAINER (type_box), 3);
 
-	gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD);
-	gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
-	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
-
-	twelvehour = gtk_radio_button_new_with_label (NULL, _("12 hour"));
-	gtk_table_attach (GTK_TABLE (table), twelvehour, 0, 1, 0, 1,
-			  GTK_FILL, GTK_FILL | GTK_EXPAND,
-			  0, 0);
-	g_object_set_data (G_OBJECT (twelvehour), "user_data", cd);
+	twelvehour = gtk_radio_button_new_with_mnemonic (NULL, _("_12 hour"));
+	gtk_box_pack_start (GTK_BOX (type_box), twelvehour, FALSE, FALSE, 0);
 	gtk_widget_show (twelvehour);
+	g_object_set_data (G_OBJECT (twelvehour), "user_data", cd);
 
-	twentyfourhour = gtk_radio_button_new_with_label (gtk_radio_button_get_group (GTK_RADIO_BUTTON (twelvehour)),
-							  _("24 hour"));
-
-	gtk_table_attach (GTK_TABLE (table), twentyfourhour, 0, 1, 1, 2,
-			  GTK_FILL, GTK_FILL | GTK_EXPAND,
-			  0, 0);
-	g_object_set_data (G_OBJECT (twentyfourhour), "user_data", cd);
+	twentyfourhour = gtk_radio_button_new_with_mnemonic (gtk_radio_button_get_group (GTK_RADIO_BUTTON (twelvehour)),
+							     _("_24 hour"));
+	gtk_box_pack_start (GTK_BOX (type_box), twentyfourhour, FALSE, FALSE, 0);
 	gtk_widget_show (twentyfourhour);
+	g_object_set_data (G_OBJECT (twentyfourhour), "user_data", cd);
 
-	switch (cd->hourformat) {
-	case 12:
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (twelvehour),
-					      TRUE);
-		break;
-
-	case 24:
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (twentyfourhour),
-					      TRUE);
-		break;
-	}
+	unixtime = gtk_radio_button_new_with_mnemonic (gtk_radio_button_get_group (GTK_RADIO_BUTTON (twelvehour)), _("_UNIX time"));
+	gtk_box_pack_start (GTK_BOX (type_box), unixtime, FALSE, FALSE, 0);
+	gtk_widget_show (unixtime);
 
+	internettime = gtk_radio_button_new_with_mnemonic (gtk_radio_button_get_group (GTK_RADIO_BUTTON (twelvehour)), _("_Internet time"));
+	gtk_box_pack_start (GTK_BOX (type_box), internettime, FALSE, FALSE, 0);
+	gtk_widget_show (internettime);
+   
+	if (cd->unixtime)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (unixtime), TRUE);
+	else if (cd->internettime)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (internettime), TRUE);
+	else if (cd->hourformat == 12)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (twelvehour), TRUE);
+	else if (cd->hourformat == 24)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (twentyfourhour), TRUE);
+   
+	g_signal_connect (G_OBJECT (unixtime), "toggled",
+			  G_CALLBACK (set_unixtime_cb),
+			  cd);
+	g_signal_connect (G_OBJECT (internettime), "toggled",
+			  G_CALLBACK (set_internettime_cb),
+			  cd);
 	g_signal_connect (G_OBJECT (twelvehour), "toggled",
 			  G_CALLBACK (set_hour_format_cb),
 			  GINT_TO_POINTER (12));
 	g_signal_connect (G_OBJECT (twentyfourhour), "toggled",
 			  G_CALLBACK (set_hour_format_cb),
 			  GINT_TO_POINTER (24));
-	
-	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
-	gtk_box_pack_start_defaults (GTK_BOX (hbox), vbox);
-	gtk_widget_show (vbox);
 
-	showdate = gtk_check_button_new_with_label (_("Show date in applet"));
-	gtk_box_pack_start_defaults (GTK_BOX (vbox), showdate);
+	options_frame = gtk_frame_new ("");
+	gtk_box_pack_start (GTK_BOX (hbox), options_frame, FALSE, FALSE, 0);
+	gtk_widget_show (options_frame);
+	gtk_container_set_border_width (GTK_CONTAINER (options_frame), 3);
+	gtk_frame_set_shadow_type (GTK_FRAME (options_frame), GTK_SHADOW_NONE);
+	
+	vbox = gtk_vbox_new (FALSE, 3);
+	gtk_container_add (GTK_CONTAINER (options_frame), vbox);
+	gtk_widget_show (vbox);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 3);
+
+	showseconds = gtk_check_button_new_with_mnemonic (_("Show _seconds"));
+	gtk_box_pack_start (GTK_BOX (vbox), showseconds, FALSE, FALSE, 0);
+	gtk_widget_show (showseconds);
+	
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (showseconds),
+	                              cd->showseconds);
+	g_signal_connect (G_OBJECT (showseconds), "toggled",
+			  G_CALLBACK (set_show_seconds_cb),
+			  cd);	   
+
+	showdate = gtk_check_button_new_with_mnemonic (_("Show _date"));
+	gtk_box_pack_start (GTK_BOX (vbox), showdate, FALSE, FALSE, 0);
 	gtk_widget_show (showdate);
 	
-	if (cd->showdate)
-	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (showdate),
-					TRUE);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (showdate),
+	                              cd->showdate);
 	g_signal_connect (G_OBJECT (showdate), "toggled",
 			  G_CALLBACK (set_show_date_cb),
 			  cd);	   
 
-	showtooltip = gtk_check_button_new_with_label (_("Show date in tooltip"));
-	gtk_box_pack_start_defaults (GTK_BOX (vbox), showtooltip);
-	gtk_widget_show (showtooltip);
-	
-	if (cd->showtooltip)
-	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (showtooltip),
-					TRUE);
-	g_signal_connect (G_OBJECT (showtooltip), "toggled",
-			  G_CALLBACK (set_show_tooltip_cb),
-			  cd);	
-
-	use_gmt_time = gtk_check_button_new_with_label (_("Use GMT"));
-	gtk_box_pack_start_defaults (GTK_BOX (vbox), use_gmt_time);
+	use_gmt_time = gtk_check_button_new_with_mnemonic (_("Use _UTC"));
+	gtk_box_pack_start (GTK_BOX (vbox), use_gmt_time, FALSE, FALSE, 0);
 	gtk_widget_show (use_gmt_time);
 	
-	if (cd->gmt_time)
-	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (use_gmt_time),
-					TRUE);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (use_gmt_time),
+	                              cd->gmt_time);
 	g_signal_connect (G_OBJECT (use_gmt_time), "toggled",
 			  G_CALLBACK (set_gmt_time_cb),
 			  cd);	
 
-	unixtime = gtk_check_button_new_with_label (_("Unix time"));
-	gtk_box_pack_start_defaults (GTK_BOX (vbox), unixtime);
-	gtk_widget_show (unixtime);
-
-        internettime = gtk_check_button_new_with_label (_("Internet time"));
-	gtk_box_pack_start_defaults (GTK_BOX (vbox), internettime);
-	gtk_widget_show (internettime);
-   
-        g_signal_connect (G_OBJECT (unixtime), "toggled",
+	/* Some combinations of options do not make sense */
+	if (cd->unixtime) {
+		gtk_widget_set_sensitive (showseconds, FALSE);
+		gtk_widget_set_sensitive (showdate, FALSE);
+		gtk_widget_set_sensitive (use_gmt_time, FALSE);
+	}
+	if (cd->internettime) {
+		gtk_widget_set_sensitive (showdate, FALSE);
+		gtk_widget_set_sensitive (use_gmt_time, FALSE);
+	}
+	g_signal_connect (G_OBJECT (unixtime), "toggled",
 			  G_CALLBACK (set_datasensitive_cb),
-			  hour_frame);
+			  showseconds);
 	g_signal_connect (G_OBJECT (unixtime), "toggled",
 			  G_CALLBACK (set_datasensitive_cb),
 			  showdate);
 	g_signal_connect (G_OBJECT (unixtime), "toggled",
 			  G_CALLBACK (set_datasensitive_cb),
 			  use_gmt_time);
-	g_signal_connect (G_OBJECT (unixtime), "toggled",
-			  G_CALLBACK (set_datasensitive_cb),
-			  internettime);
-			   
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (unixtime),
-				      cd->unixtime);
-
-/* internet time */
-	g_signal_connect (G_OBJECT (internettime), "toggled",
-			  G_CALLBACK (set_datasensitive_cb),
-			  hour_frame);
 	g_signal_connect (G_OBJECT (internettime), "toggled",
 			  G_CALLBACK (set_datasensitive_cb),
 			  showdate);
 	g_signal_connect (G_OBJECT (internettime), "toggled",
 			  G_CALLBACK (set_datasensitive_cb),
 			  use_gmt_time);   
-	g_signal_connect (G_OBJECT (internettime), "toggled",
-			  G_CALLBACK (set_datasensitive_cb),
-			  unixtime);   
    
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (internettime),
-				      cd->internettime);
-   
-	g_signal_connect (G_OBJECT (unixtime), "toggled",
-			  G_CALLBACK (set_unixtime_cb),
-			    cd);
-	g_signal_connect (G_OBJECT (internettime), "toggled",
-			  G_CALLBACK (set_internettime_cb),
-			  cd);
-
-	gtk_widget_show (hour_frame);
-	gtk_widget_show (hbox);
-
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (cd->props)->vbox), hbox, FALSE, FALSE, 0);
-
-	
 	g_signal_connect (G_OBJECT (cd->props), "destroy",
 			  G_CALLBACK (gtk_widget_destroyed), &(cd->props));
 	g_signal_connect (G_OBJECT (cd->props), "response",
 			  G_CALLBACK (properties_response_cb), NULL);
+
+	/* sets up atk relation  */
+	list = g_slist_append (NULL, twelvehour);
+	list = g_slist_append (list, twentyfourhour);
+	add_atk_relation (use_gmt_time, list, ATK_RELATION_CONTROLLED_BY);
+	g_slist_free (list);
+
+	list = g_slist_append (NULL, use_gmt_time);
+	add_atk_relation (twelvehour, list, ATK_RELATION_CONTROLLER_FOR);
+	add_atk_relation (twentyfourhour, list, ATK_RELATION_CONTROLLER_FOR);
+	g_slist_free (list);
 				
 	gtk_widget_show (cd->props);
 }
@@ -1048,18 +1047,22 @@ display_about_dialog (BonoboUIComponent *uic,
 		      const gchar       *verbname)
 {
 	static GtkWidget *about = NULL;
-	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf = NULL;
 	gchar *file;
 	
 	static const gchar *authors[] =
 	{
-		"Miguel de Icaza <miguel@kernel.org>",
-		"Federico Mena <quartic@gimp.org>",
-		"Stuart Parmenter <pavlov@innerx.net>",
 		"George Lebl <jirka@5z.com>",
-		"Alexander Larsson <alla@lysator.liu.se>",
+		"Gediminas Paulauskas <menesis@delfi.lt>",
 		NULL
 	};
+	static const char *documenters[] =
+	{
+		"Dan Mueth (d-mueth@uchicago.edu)",
+		NULL
+	};
+	/* Translator credits */
+	const char *translator_credits = _("translator_credits");
 
 	if (about != NULL)
 	{
@@ -1069,18 +1072,20 @@ display_about_dialog (BonoboUIComponent *uic,
 
 	pixbuf = NULL;
 	
-	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-clock.png", TRUE, NULL);
-	if (!file) {
-		g_warning (G_STRLOC ": gnome-clock.png cannot be found");
+	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
+	                                  "gnome-clock.png", TRUE, NULL);
+	if (file) {
 		pixbuf = gdk_pixbuf_new_from_file (file, NULL);
-	}
+		g_free (file);
+	} else
+		g_warning (G_STRLOC ": gnome-clock.png cannot be found");
 
-	about = gnome_about_new (_("Clock Applet"), "1.0",
-				 _("(c) 1998-2001 the Free Software Foundation"),
-				 _("The clock applet gives your panel a lightweight and simple display of the date and time"),
+	about = gnome_about_new (_("Clock Applet"), VERSION,
+				 _("(C) 1998-2002 the Free Software Foundation"),
+				 _("The Clock Applet displays current time and date"),
 				 authors,
-				 NULL, /* documenters */
-				 NULL, /* translator_credits */
+				 documenters,
+				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 				 pixbuf);
 	
 	gtk_window_set_wmclass (GTK_WINDOW (about), "clock", "Clock");
